#ifndef __DEVICE_HH___
#define __DEVICE_HH___

#include <qpixmap.h>
#include <qcheckbox.h>
#include <qvbox.h>

class DeviceManager;
class ProfileManager;

namespace network_config {
  class InterfaceSection;
}


class Device : public QObject
{
  Q_OBJECT
public:
  Device (DeviceManager* device_manager, ProfileManager* profile_manager);

  virtual QPixmap* GetEnabledPicture () = 0;
  virtual QPixmap* GetDisabledPicture () = 0;
  virtual const QString& GetName () = 0;
  virtual const QString& GetTitle () = 0;
  virtual QWidget* CreateStatus (QWidget* parent) { return 0; }

  bool isEnabled () { return enabled; }
  bool isVisible () { return visible; }

  void Build (QVBox* box);

public slots:

  // connected to own checkbox
  void checkChanged (bool state);
  // connected to profileManager
  void newConfig ();
  void newActive ();
protected:
  void setEnabled (bool state); // not recommended to call this directly
  void setVisible (bool state); // show / hide status widget

  virtual void BuildContent (QVBox* box);
  virtual void RefreshContent ();  // invoked by newConfig ()
  virtual void EnabledToggled (); // invoked by checkChanged ()
  virtual void ActiveToggled (bool state); // invoked by newActive ();

  network_config::InterfaceSection* config;

  DeviceManager* m_device_manager;
  ProfileManager* m_profile_manager;

  bool auto_up;
private:
  bool visible;
  bool enabled;

  QCheckBox* enable_check;
};

#endif // __DEVICE_HH___
