#include "main_window.hh"
//#include <kapp.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <kseparator.h>
#include <qpushbutton.h>
#include <kmessagebox.h>

#include "netconf_app.hh"
#include "device_status.hh"
#include "device_list.hh"
#include "device_manager.hh"
#include "device_factory.hh"
#include "profile_switch.hh"
#include "profile_manager.hh"
#include "profilename_dialog.hh"

const bool enable_tree_dump = true;

MainWindow::MainWindow (const char * name, NetconfApp* application ) : KMainWindow ( 0L, name, 0 )
{
  m_application = application;

  setCaption (i18n("Network Config"));

  QPopupMenu *filemenu = new QPopupMenu;
  filemenu->insertItem( i18n( "&Save Config"), m_application -> profile_manager, SLOT(SaveConfig ()));
  filemenu->insertItem( i18n( "&Revert Config"), m_application -> profile_manager, SLOT(RevertConfig ()));
  if (enable_tree_dump)
    filemenu->insertItem( "&Debug", m_application -> profile_manager, SLOT(DebugConfig ()));
  filemenu->insertItem( i18n( "&Quit" ), kapp, SLOT(quit()) );


  QPopupMenu *profilemenu = new QPopupMenu;
  profilemenu->insertItem( i18n( "&Add new profile" ), this, SLOT(AddProfileRequest()));
  profilemenu->insertItem( i18n( "&Delete current" ), this, SLOT(DeleteProfileRequest()));
  profilemenu->insertItem( i18n( "&Rename profile" ), this, SLOT(RenameProfileRequest()));
  
  QPopupMenu *devicemenu = new QPopupMenu;
  devicemenu->insertItem( i18n( "&Scan for new serial modems" ), m_application -> profile_manager, SLOT(ReScanSerials ()));


  QString about =
    i18n("T2 Network Configuration 0.0.1\n\n"
	"(C) 2005 Valentin Ziegler\n"
	"<valentin@exactcode.de>\n"
	"This application comes with ABSOLUTELY NO WARRANTY\n"
	"This is free software, and you are welcome to redistribute it\n"
	"under certain conditions\n");

  QPopupMenu *helpmenu = helpMenu( about );
  KMenuBar *menu = menuBar();
  menu->insertItem( i18n( "&File" ), filemenu);
  menu->insertItem( i18n( "&Profiles" ) , profilemenu);
  menu->insertItem( i18n( "&Devices" ), devicemenu);
  menu->insertSeparator();
  menu->insertItem( i18n( "&Help" ), helpmenu);


  vbox = new QVBox ( this );

  current_active = new QLabel (vbox);
  slotNewActive ();

  // active_label -> setText ("Aktives Profil: (keins)");
 
  //  QHBox* hpppstat = new QHBox (vbox);
  //QPushButton* ppp_icon = new QPushButton (hpppstat);
  //QLabel* tmp_icon = new QLabel (hpppstat);
  //ppp_icon -> setPixmap (*( new QPixmap ("/home/valentin/netconf-trunk/gui/pixmaps/telephone_receiver.png")));
  //ppp_icon -> setPixmap (*( new QPixmap ("/home/valentin/netconf-trunk/gui/pixmaps/telephone.png")));
  //tmp_icon -> setPixmap (*( new QPixmap ("/home/valentin/netconf-trunk/gui/pixmaps/wlan-small.png")));
  //ppp_icon -> setMaximumWidth (32);
  //  tmp_icon -> setMaximumWidth (32);

  //  QLabel* ppp_stat = new QLabel (hpppstat);
  //ppp_stat -> setText (" ppp0 online seit 01:37:22");
  //ppp_stat -> setText (" ppp0 offline");
  //  ppp_stat -> setText (" eth1 Verbindungsqualitt: 93%");
  QHBoxLayout* l ;//= (QHBoxLayout*) hpppstat-> layout ();
  //l ->   insertStretch (2, 0);

  dstatus = new DeviceStatus (vbox, m_application -> device_manager );
  m_application -> device_manager -> devstatus = dstatus;  

  pswitch = new ProfileSwitch (vbox, m_application -> profile_manager);

  new KSeparator (KSeparator::HLine,  vbox);

  dlist = new DeviceList ( vbox, m_application -> device_manager);
  m_application -> device_manager -> devlist = dlist;

  DeviceFactory::CreateSystemDevices (m_application -> device_manager, m_application -> profile_manager);

  QHBox* hactions = new QHBox (vbox);
  l = (QHBoxLayout*) hactions-> layout ();
  l ->   addStretch (0);
  //  ok = new QPushButton (i18n("Ok"), hactions);
  apply = new QPushButton (i18n("Save"), hactions);
  cancel = new QPushButton ("Cancel", hactions);

  QObject::connect ( m_application -> profile_manager, SIGNAL ( changedActiveProfile () ), this, SLOT ( slotNewActive () ) );
  //  QObject::connect (ok, SIGNAL (clicked()), this, SLOT(Ok()));
  QObject::connect (apply, SIGNAL (clicked()), this, SLOT(Apply()));
  QObject::connect (cancel, SIGNAL (clicked()), this, SLOT(Cancel()));

  profile_name_dialog = new ProfileNameDialog (this);

  setCentralWidget( vbox );
}


MainWindow::~MainWindow ()
{

}

void MainWindow::slotNewActive ()
{
  QString text = i18n("active profile: ");
  int active_index = m_application -> profile_manager -> ActiveProfileIndex ();
  if (active_index < 0)
    text += i18n("none");
  else
    text += m_application -> profile_manager -> getI18NProfileName(active_index);
  current_active -> setText (text);
}


void MainWindow::closeEvent ( QCloseEvent * e )
{
  e->accept();
}

void MainWindow::Ok ()
{
  m_application -> profile_manager -> SaveConfig ();
  hide ();
}

void MainWindow::Apply ()
{
  ProfileManager* m = m_application -> profile_manager;
  if (m -> ActiveProfileIndex () == m -> ConfigProfileIndex ()) {
    KMessageBox::information (this, i18n("Changes for active profiles do not take place before profile is re-activated.")); 
  }
  m_application -> profile_manager -> SaveConfig ();
}

void MainWindow::Cancel ()
{
  m_application -> profile_manager -> RevertConfig ();
  hide ();
}

void MainWindow::AddProfileRequest ()
{
  profile_name_dialog -> SetProposal ("Profile01");
  if (profile_name_dialog -> exec ()) {
    std::string name = profile_name_dialog -> Content().ascii();
    if (CheckDuplicate (name))
      m_application -> profile_manager -> AddProfile (name);
  }
}

void MainWindow::DeleteProfileRequest ()
{
  unsigned int current_index = m_application -> profile_manager -> ConfigProfileIndex ();
  if (m_application -> profile_manager -> Profiles()[current_index] == "default") {
    PopupSorryNoDefaultMessing ();
    std::cout << "refusing to remove default profile" << std::endl;
    return;
  }
  m_application -> profile_manager -> RemoveProfile (current_index);
}

void MainWindow::RenameProfileRequest ()
{
  unsigned int current_index = m_application -> profile_manager -> ConfigProfileIndex ();
  if (m_application -> profile_manager -> Profiles()[current_index] == "default") {
    PopupSorryNoDefaultMessing ();
    std::cout << "refusing to rename default profile" << std::endl;
    return;
  }

  if (m_application -> profile_manager -> ActiveProfileIndex () == (int) current_index) {
    KMessageBox::sorry (this, i18n("Sorry, activated profiles cannot be renamed. Please deactive the current profile first.")); 
    std::cout << "refusing to rename active profile" << std::endl;
    return;
  }

  std::string current = m_application -> profile_manager -> Profiles()[current_index];
  profile_name_dialog -> SetProposal (current.c_str());
  if (profile_name_dialog -> exec ()) {
    std::string name = profile_name_dialog -> Content().ascii();
    if (CheckDuplicate (name, current) && name != current)
      m_application -> profile_manager -> RenameProfile (current, name);
  } 
}


bool MainWindow::CheckDuplicate (const std::string& new_name, const std::string& tolerate)
{
  unsigned int no_profiles = m_application -> profile_manager -> Profiles().size();
  bool valid = true;
  for (unsigned int i = 0; i < no_profiles && valid; i++)
    if (new_name != tolerate && new_name == m_application -> profile_manager -> Profiles()[i])
      valid = false;
  if (!valid) {
    QString text = i18n("Sorry, profile") + QString(" ");
    text += new_name.c_str() + QString(" ") + i18n("already exists.");
    KMessageBox::sorry (this, text); 
    std::cout << "new name confilcts with old ones" << std::endl;
  }
  return valid;
}

void MainWindow::PopupSorryNoDefaultMessing ()
{
  KMessageBox::sorry (this, i18n("Sorry, the 'default' profile is mandatory and cannot be removed or renamed.")); 
}
