//#include <vector>
//#include <string>
#include <qobject.h>
#include <qstring.h>
#include <klocale.h>

#include "network_manager.hh"
#include "devices.hh"


class ProfileManager : public QObject
{
  Q_OBJECT
public:
  typedef std::string profile_id;
  typedef std::string device_id;
  typedef QString serial_id;

  ProfileManager ();
  ~ProfileManager ();

  unsigned int AddProfile (const profile_id& id);
  void RenameProfile (const profile_id& id_old, const profile_id& id_new);
  void RemoveProfile (unsigned int profile_index);

  // translates 'default' into language default 
  QString getI18NProfileName (const profile_id& id);
  QString getI18NProfileName (unsigned int profile_index) { return getI18NProfileName (profiles[profile_index]);}

  const std::vector <profile_id>& Profiles () {return profiles;}
  const std::vector <device_id>& Devices () {return devices;}

  // contains devices translated into language description
  //    "USB serial <number>" or "USB <device info>"
  //    or "COM <number + 1> (optional attached device info)"
  const std::vector <serial_id>& Serials () {return serials;}

  EtherDev* getInfo (unsigned int device_index); // returns 0 for logical devices
  SerialDev* getSerial (unsigned int serial_index);

  // methods return -1 if not applicable
  int ActiveProfileIndex () {return active_profile_index;}
  int ConfigProfileIndex () {return config_profile_index;}

  // methods return 0 if not applicable
  network_config::InterfaceSection* getSection (int profile_index, const device_id& device);
  network_config::InterfaceSection* getActive (const device_id& device) { return getSection (active_profile_index, device);}
  network_config::InterfaceSection* getConfig (const device_id& device) { return getSection (config_profile_index, device);}

  void ActivateProfile (unsigned int profile_index);
  void DeactivateProfile ();
  void ConfigureProfile (unsigned int profile_index);

  bool interfaceIsUp (const device_id& device);
  void setInterfaceUp (const device_id& device, bool status);


public slots:
  void LoadConfig (); // not able to re-load right now !
  void SaveConfig ();
  void RevertConfig (); // dito
  void DebugConfig ();
  void ReScanSerials (); // also sorts in all devices found in the config and remembers devices found before


signals:
  void changedActiveProfile ();
  void changedConfigProfile ();
  void newProfileList (); // indexes may change too !
  void interfaceDown (const device_id& device); // specific interface went down without ifdown call (not used yet)
  void newSerialList ();

private:
  int FindIndex (const profile_id& name);

  network_config::ConfigManager* m_manager;

  std::vector <profile_id> profiles;
  std::vector <device_id> devices;
  std::vector <serial_id> serials;

  EtherDevList ether_devs;
  SerialDevList serial_devs;

  int active_profile_index;
  int config_profile_index;

  bool signal_lock;
};
